@@ -1,11 +1,10 @@
+use crate::prelude::*;
 use arrow::array::growable::make_growable;
-use arrow::array::{Array, ArrayRef};
 use arrow::error::{Error as ArrowError, Result};
-use std::sync::Arc;
 
 /// Concatenate multiple [Array] of the same type into a single [`Array`].
 /// This does not check the arrays types.
-pub fn concatenate_owned_unchecked(arrays: &[ArrayRef]) -> Result<Arc<dyn Array>> {
+pub fn concatenate_owned_unchecked(arrays: &[ArrayRef]) -> Result<ArrayRef> {
     if arrays.is_empty() {
         return Err(ArrowError::InvalidArgumentError(
             "concat requires input of at least one array".to_string(),
@@ -26,5 +25,5 @@ pub fn concatenate_owned_unchecked(arrays: &[ArrayRef]) -> Result<Arc<dyn Array>
         mutable.extend(i, 0, *len)
     }
 
-    Ok(mutable.as_arc())
+    Ok(mutable.as_box())
 }
