@@ -70,7 +70,7 @@ where
                 .zip(rhs.downcast_iter())
                 .map(|(lhs, rhs)| Box::new(kernel(lhs, rhs)) as ArrayRef)
                 .collect();
-            lhs.copy_with_chunks(chunks)
+            lhs.copy_with_chunks(chunks, false)
         }
         // broadcast right path
         (_, 1) => {
