@@ -1,5 +1,6 @@
 use super::*;
 use crate::chunked_array::ops::append::new_chunks;
+use crate::series::IsSorted;
 
 impl CategoricalChunked {
     pub fn append(&mut self, other: &Self) -> Result<()> {
@@ -8,6 +9,7 @@ impl CategoricalChunked {
 
         let len = self.len();
         new_chunks(&mut self.logical.chunks, &other.logical().chunks, len);
+        self.logical.set_sorted2(IsSorted::Not);
         Ok(())
     }
 }
