@@ -462,9 +462,9 @@ impl ChunkAgg<IdxSize> for BooleanChunked {
                 self.downcast_iter()
                     .map(|arr| match arr.validity() {
                         Some(validity) => {
-                            (arr.len() - (validity & arr.values()).null_count()) as IdxSize
+                            (arr.len() - (validity & arr.values()).unset_bits()) as IdxSize
                         }
-                        None => (arr.len() - arr.values().null_count()) as IdxSize,
+                        None => (arr.len() - arr.values().unset_bits()) as IdxSize,
                     })
                     .sum(),
             )
@@ -827,7 +827,7 @@ mod test {
     #[test]
     fn test_var() {
         // validated with numpy
-        // Note that numpy as an argument ddof wich influences results. The default is ddof=0
+        // Note that numpy as an argument ddof which influences results. The default is ddof=0
         // we chose ddof=1, which is standard in statistics
         let ca1 = Int32Chunked::new("", &[5, 8, 9, 5, 0]);
         let ca2 = Int32Chunked::new(
