@@ -57,8 +57,8 @@ impl private::PrivateSeries for SeriesWrap<DatetimeChunked> {
             .into_series()
     }
 
-    fn _set_sorted(&mut self, reverse: bool) {
-        self.0.deref_mut().set_sorted(reverse)
+    fn _set_sorted(&mut self, is_sorted: IsSorted) {
+        self.0.deref_mut().set_sorted2(is_sorted)
     }
 
     #[cfg(feature = "zip_with")]
@@ -332,7 +332,7 @@ impl SeriesTrait for SeriesWrap<DatetimeChunked> {
         let mut out = ChunkTake::take_unchecked(self.0.deref(), idx.into());
 
         if self.0.is_sorted() && (idx.is_sorted() || idx.is_sorted_reverse()) {
-            out.set_sorted(idx.is_sorted_reverse())
+            out.set_sorted2(idx.is_sorted2())
         }
 
         Ok(out
