@@ -108,7 +108,6 @@ impl SeriesTrait for SeriesWrap<StructChunked> {
             .into_series()
     }
 
-    #[doc(hidden)]
     fn append(&mut self, other: &Series) -> Result<()> {
         let other = other.struct_()?;
         let offset = self.chunks().len();
@@ -125,7 +124,6 @@ impl SeriesTrait for SeriesWrap<StructChunked> {
         Ok(())
     }
 
-    #[doc(hidden)]
     fn extend(&mut self, other: &Series) -> Result<()> {
         let other = other.struct_()?;
 
@@ -273,7 +271,7 @@ impl SeriesTrait for SeriesWrap<StructChunked> {
                         _ => {}
                     }
                     if let Some(validity) = &validity_agg {
-                        null_count += validity.null_count()
+                        null_count += validity.unset_bits()
                     }
                 }
             }
