@@ -3,7 +3,7 @@ use std::convert::TryFrom;
 use std::ptr::NonNull;
 
 /// A wrapper type that should make it a bit more clear that we should not clone Series
-#[derive(Debug)]
+#[derive(Debug, Copy, Clone)]
 #[cfg(feature = "private")]
 pub struct UnstableSeries<'a> {
     // A series containing a single chunk ArrayRef
@@ -42,10 +42,6 @@ impl<'a> UnstableSeries<'a> {
         }
     }
 
-    pub fn clone(&self) {
-        panic!("don't clone this type, use deep_clone")
-    }
-
     pub fn deep_clone(&self) -> Series {
         let array_ref = self.container.chunks()[0].clone();
         Series::try_from((self.container.name(), array_ref)).unwrap()
