@@ -6,9 +6,31 @@ use crate::prelude::*;
 use crate::RowCount;
 use arrow::io::parquet::read;
 use polars_core::prelude::*;
+#[cfg(feature = "serde")]
+use serde::{Deserialize, Serialize};
 use std::io::{Read, Seek};
 use std::sync::Arc;
 
+#[derive(Copy, Clone, Debug)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+pub enum ParallelStrategy {
+    /// Don't parallelize
+    None,
+    /// Parallelize over the row groups
+    Columns,
+    /// Parallelize over the columns
+    RowGroups,
+    /// Automatically determine over which unit to parallelize
+    /// This will choose the most occurring unit.
+    Auto,
+}
+
+impl Default for ParallelStrategy {
+    fn default() -> Self {
+        ParallelStrategy::Auto
+    }
+}
+
 /// Read Apache parquet format into a DataFrame.
 #[must_use]
 pub struct ParquetReader<R: Read + Seek> {
@@ -17,14 +39,14 @@ pub struct ParquetReader<R: Read + Seek> {
     n_rows: Option<usize>,
     columns: Option<Vec<String>>,
     projection: Option<Vec<usize>>,
-    parallel: bool,
+    parallel: ParallelStrategy,
     row_count: Option<RowCount>,
 }
 
 impl<R: MmapBytesReader> ParquetReader<R> {
     #[cfg(feature = "lazy")]
     // todo! hoist to lazy crate
-    pub fn finish_with_scan_ops(
+    pub fn _finish_with_scan_ops(
         mut self,
         predicate: Option<Arc<dyn PhysicalIoExpr>>,
         aggregate: Option<&[ScanAggregation]>,
@@ -55,7 +77,7 @@ impl<R: MmapBytesReader> ParquetReader<R> {
     }
 
     /// Read the parquet file in parallel (default). The single threaded reader consumes less memory.
-    pub fn read_parallel(mut self, parallel: bool) -> Self {
+    pub fn read_parallel(mut self, parallel: ParallelStrategy) -> Self {
         self.parallel = parallel;
         self
     }
@@ -95,6 +117,7 @@ impl<R: MmapBytesReader> ParquetReader<R> {
 }
 
 impl<R: MmapBytesReader> SerReader<R> for ParquetReader<R> {
+    /// Create a new [`ParquetReader`] from an existing `Reader`.
     fn new(reader: R) -> Self {
         ParquetReader {
             reader,
@@ -102,7 +125,7 @@ impl<R: MmapBytesReader> SerReader<R> for ParquetReader<R> {
             n_rows: None,
             columns: None,
             projection: None,
-            parallel: true,
+            parallel: Default::default(),
             row_count: None,
         }
     }
