@@ -1,8 +1,12 @@
-#[cfg(any(feature = "ipc", feature = "parquet", feature = "avro"))]
+#[cfg(any(
+    feature = "ipc",
+    feature = "ipc_streaming",
+    feature = "parquet",
+    feature = "avro"
+))]
 use crate::ArrowSchema;
 use dirs::home_dir;
 use polars_core::frame::DataFrame;
-#[cfg(any(feature = "ipc", feature = "avro", feature = "parquet"))]
 use polars_core::prelude::*;
 use std::path::{Path, PathBuf};
 
@@ -18,7 +22,12 @@ pub fn resolve_homedir(path: &Path) -> PathBuf {
     path.into()
 }
 
-#[cfg(any(feature = "ipc", feature = "parquet", feature = "avro"))]
+#[cfg(any(
+    feature = "ipc",
+    feature = "ipc_streaming",
+    feature = "parquet",
+    feature = "avro"
+))]
 pub(crate) fn apply_projection(schema: &ArrowSchema, projection: &[usize]) -> ArrowSchema {
     let fields = &schema.fields;
     let fields = projection
@@ -28,7 +37,12 @@ pub(crate) fn apply_projection(schema: &ArrowSchema, projection: &[usize]) -> Ar
     ArrowSchema::from(fields)
 }
 
-#[cfg(any(feature = "ipc", feature = "avro", feature = "parquet"))]
+#[cfg(any(
+    feature = "ipc",
+    feature = "ipc_streaming",
+    feature = "avro",
+    feature = "parquet"
+))]
 pub(crate) fn columns_to_projection(
     columns: Vec<String>,
     schema: &ArrowSchema,
@@ -69,7 +83,7 @@ pub(crate) fn columns_to_projection(
 
 /// Because of threading every row starts from `0` or from `offset`.
 /// We must correct that so that they are monotonically increasing.
-pub(crate) fn update_row_counts(dfs: &mut [(DataFrame, u32)]) {
+pub(crate) fn update_row_counts(dfs: &mut [(DataFrame, IdxSize)]) {
     if !dfs.is_empty() {
         let mut previous = dfs[0].1;
         for (df, n_read) in &mut dfs[1..] {
