@@ -1,5 +1,4 @@
 use super::*;
-use polars_core::prelude::DataType::{Datetime, Duration};
 use polars_time::prelude::TemporalMethods;
 
 /// Specialized expressions for [`Series`] with dates/datetimes.
@@ -34,8 +33,8 @@ impl DateLikeNameSpace {
                 )),
             },
             GetOutput::map_dtype(move |dtype| match dtype {
-                DataType::Duration(_) => Duration(tu),
-                DataType::Datetime(_, tz) => Datetime(tu, tz.clone()),
+                DataType::Duration(_) => DataType::Duration(tu),
+                DataType::Datetime(_, tz) => DataType::Datetime(tu, tz.clone()),
                 _ => panic!("expected duration or datetime"),
             }),
         )
@@ -69,7 +68,7 @@ impl DateLikeNameSpace {
         let function = move |s: Series| s.year().map(|ca| ca.into_series());
         self.0
             .map(function, GetOutput::from_type(DataType::UInt32))
-            .with_fmt("year")
+            .with_fmt("dt.year")
     }
 
     /// Get the month of a Date/Datetime
@@ -77,8 +76,18 @@ impl DateLikeNameSpace {
         let function = move |s: Series| s.month().map(|ca| ca.into_series());
         self.0
             .map(function, GetOutput::from_type(DataType::UInt32))
-            .with_fmt("month")
+            .with_fmt("dt.month")
     }
+
+    /// Extract quarter from underlying NaiveDateTime representation.
+    /// Quarters range from 1 to 4.
+    pub fn quarter(self) -> Expr {
+        let function = move |s: Series| s.quarter().map(|ca| ca.into_series());
+        self.0
+            .map(function, GetOutput::from_type(DataType::UInt32))
+            .with_fmt("dt.quarter")
+    }
+
     /// Extract the week from the underlying Date representation.
     /// Can be performed on Date and Datetime
 
@@ -88,7 +97,7 @@ impl DateLikeNameSpace {
         let function = move |s: Series| s.week().map(|ca| ca.into_series());
         self.0
             .map(function, GetOutput::from_type(DataType::UInt32))
-            .with_fmt("week")
+            .with_fmt("dt.week")
     }
 
     /// Extract the week day from the underlying Date representation.
@@ -99,7 +108,7 @@ impl DateLikeNameSpace {
         let function = move |s: Series| s.weekday().map(|ca| ca.into_series());
         self.0
             .map(function, GetOutput::from_type(DataType::UInt32))
-            .with_fmt("weekday")
+            .with_fmt("dt.weekday")
     }
 
     /// Get the month of a Date/Datetime
@@ -114,21 +123,21 @@ impl DateLikeNameSpace {
         let function = move |s: Series| s.ordinal_day().map(|ca| ca.into_series());
         self.0
             .map(function, GetOutput::from_type(DataType::UInt32))
-            .with_fmt("ordinal_day")
+            .with_fmt("dt.ordinal_day")
     }
     /// Get the hour of a Datetime/Time64
     pub fn hour(self) -> Expr {
         let function = move |s: Series| s.hour().map(|ca| ca.into_series());
         self.0
             .map(function, GetOutput::from_type(DataType::UInt32))
-            .with_fmt("hour")
+            .with_fmt("dt.hour")
     }
     /// Get the minute of a Datetime/Time64
     pub fn minute(self) -> Expr {
         let function = move |s: Series| s.minute().map(|ca| ca.into_series());
         self.0
             .map(function, GetOutput::from_type(DataType::UInt32))
-            .with_fmt("minute")
+            .with_fmt("dt.minute")
     }
 
     /// Get the second of a Datetime/Time64
@@ -136,14 +145,14 @@ impl DateLikeNameSpace {
         let function = move |s: Series| s.second().map(|ca| ca.into_series());
         self.0
             .map(function, GetOutput::from_type(DataType::UInt32))
-            .with_fmt("second")
+            .with_fmt("dt.second")
     }
     /// Get the nanosecond of a Time64
     pub fn nanosecond(self) -> Expr {
         let function = move |s: Series| s.nanosecond().map(|ca| ca.into_series());
         self.0
             .map(function, GetOutput::from_type(DataType::UInt32))
-            .with_fmt("nanosecond")
+            .with_fmt("dt.nanosecond")
     }
 
     pub fn timestamp(self, tu: TimeUnit) -> Expr {
@@ -152,6 +161,14 @@ impl DateLikeNameSpace {
                 move |s| s.timestamp(tu).map(|ca| ca.into_series()),
                 GetOutput::from_type(DataType::Int64),
             )
-            .with_fmt("timestamp")
+            .with_fmt("dt.timestamp")
+    }
+
+    /// Offset this `Date/Datetime` by a given offset [`Duration`].
+    /// This will take leap years/ months into account.
+    #[cfg(feature = "date_offset")]
+    pub fn offset_by(self, by: Duration) -> Expr {
+        self.0
+            .map_private(FunctionExpr::DateOffset(by), "dt.offset_by")
     }
 }
