@@ -1,12 +1,13 @@
 use crate::prelude::*;
 use polars_core::prelude::*;
+use polars_io::parquet::ParallelStrategy;
 use polars_io::RowCount;
 
 #[derive(Clone)]
 pub struct ScanArgsParquet {
     pub n_rows: Option<usize>,
     pub cache: bool,
-    pub parallel: bool,
+    pub parallel: ParallelStrategy,
     pub rechunk: bool,
     pub row_count: Option<RowCount>,
 }
@@ -16,7 +17,7 @@ impl Default for ScanArgsParquet {
         Self {
             n_rows: None,
             cache: true,
-            parallel: true,
+            parallel: Default::default(),
             rechunk: true,
             row_count: None,
         }
@@ -28,7 +29,7 @@ impl LazyFrame {
         path: String,
         n_rows: Option<usize>,
         cache: bool,
-        parallel: bool,
+        parallel: ParallelStrategy,
         row_count: Option<RowCount>,
         rechunk: bool,
     ) -> Result<Self> {
@@ -36,10 +37,42 @@ impl LazyFrame {
             LogicalPlanBuilder::scan_parquet(path, n_rows, cache, parallel, row_count, rechunk)?
                 .build()
                 .into();
-        lf.opt_state.agg_scan_projection = true;
+        lf.opt_state.file_caching = true;
         Ok(lf)
     }
 
+    fn concat_impl(lfs: Vec<LazyFrame>, args: ScanArgsParquet) -> Result<LazyFrame> {
+        concat(&lfs, args.rechunk).map(|mut lf| {
+            if let Some(n_rows) = args.n_rows {
+                lf = lf.slice(0, n_rows as IdxSize)
+            };
+            if let Some(rc) = args.row_count {
+                lf = lf.with_row_count(&rc.name, Some(rc.offset))
+            };
+            lf
+        })
+    }
+
+    /// Create a LazyFrame directly from a parquet scan.
+    #[cfg_attr(docsrs, doc(cfg(feature = "parquet")))]
+    pub fn scan_parquet_files(paths: Vec<String>, args: ScanArgsParquet) -> Result<Self> {
+        let lfs = paths
+            .iter()
+            .map(|p| {
+                Self::scan_parquet_impl(
+                    p.to_string(),
+                    args.n_rows,
+                    args.cache,
+                    args.parallel,
+                    None,
+                    args.rechunk,
+                )
+            })
+            .collect::<Result<Vec<_>>>()?;
+
+        Self::concat_impl(lfs, args)
+    }
+
     /// Create a LazyFrame directly from a parquet scan.
     #[cfg_attr(docsrs, doc(cfg(feature = "parquet")))]
     pub fn scan_parquet(path: String, args: ScanArgsParquet) -> Result<Self> {
@@ -54,25 +87,14 @@ impl LazyFrame {
                         path_string,
                         args.n_rows,
                         args.cache,
-                        false,
+                        ParallelStrategy::None,
                         None,
                         args.rechunk,
                     )
                 })
                 .collect::<Result<Vec<_>>>()?;
 
-            concat(&lfs, args.rechunk)
-                .map_err(|_| PolarsError::ComputeError("no matching files found".into()))
-                .map(|mut lf| {
-                    if let Some(n_rows) = args.n_rows {
-                        lf = lf.slice(0, n_rows as IdxSize)
-                    };
-
-                    if let Some(rc) = args.row_count {
-                        lf = lf.with_row_count(&rc.name, Some(rc.offset))
-                    };
-                    lf
-                })
+            Self::concat_impl(lfs, args)
         } else {
             Self::scan_parquet_impl(
                 path,
