@@ -1,3 +1,4 @@
+use crate::dsl::function_expr::FunctionExpr;
 use crate::logical_plan::Context;
 use crate::prelude::*;
 use polars_arrow::prelude::QuantileInterpolOptions;
@@ -74,12 +75,19 @@ pub enum AExpr {
         truthy: Node,
         falsy: Node,
     },
-    Function {
+    AnonymousFunction {
         input: Vec<Node>,
-        function: NoEq<Arc<dyn SeriesUdf>>,
+        function: SpecialEq<Arc<dyn SeriesUdf>>,
         output_type: GetOutput,
         options: FunctionOptions,
     },
+    Function {
+        /// function arguments
+        input: Vec<Node>,
+        /// function to apply
+        function: FunctionExpr,
+        options: FunctionOptions,
+    },
     Shift {
         input: Node,
         periods: i64,
@@ -93,8 +101,8 @@ pub enum AExpr {
     Wildcard,
     Slice {
         input: Node,
-        offset: i64,
-        length: usize,
+        offset: Node,
+        length: Node,
     },
     Count,
     Nth(i64),
@@ -117,6 +125,36 @@ impl AExpr {
             .map(|f| f.data_type().clone())
     }
 
+    pub(crate) fn replace_input(self, input: Node) -> Self {
+        use AExpr::*;
+        match self {
+            Alias(_, name) => Alias(input, name),
+            IsNotNull(_) => IsNotNull(input),
+            IsNull(_) => IsNull(input),
+            Cast {
+                expr: _,
+                data_type,
+                strict,
+            } => Cast {
+                expr: input,
+                data_type,
+                strict,
+            },
+            _ => todo!(),
+        }
+    }
+
+    pub(crate) fn get_input(&self) -> Node {
+        use AExpr::*;
+        match self {
+            Alias(input, _) => *input,
+            IsNotNull(input) => *input,
+            IsNull(input) => *input,
+            Cast { expr, .. } => *expr,
+            _ => todo!(),
+        }
+    }
+
     /// Get Field result of the expression. The schema is the input data.
     pub(crate) fn to_field(
         &self,
@@ -154,16 +192,23 @@ impl AExpr {
                 arena.get(*expr).get_type(schema, ctxt, arena)?,
             )),
             Column(name) => {
-                let field = schema.field_with_name(name).map(|f| f.clone())?;
-                Ok(field)
+                let field = schema
+                    .get_field(name)
+                    .ok_or_else(|| PolarsError::NotFound(name.to_string()));
+
+                match ctxt {
+                    Context::Default => field,
+                    Context::Aggregation => field.map(|mut field| {
+                        let dtype = DataType::List(Box::new(field.data_type().clone()));
+                        field.coerce(dtype);
+                        field
+                    }),
+                }
             }
             Literal(sv) => Ok(Field::new("literal", sv.get_datatype())),
             BinaryExpr { left, right, op } => {
                 use DataType::*;
 
-                let left_type = arena.get(*left).get_type(schema, ctxt, arena)?;
-                let right_type = arena.get(*right).get_type(schema, ctxt, arena)?;
-
                 let expr_type = match op {
                     Operator::Lt
                     | Operator::Gt
@@ -173,15 +218,24 @@ impl AExpr {
                     | Operator::LtEq
                     | Operator::GtEq
                     | Operator::Or => DataType::Boolean,
-                    Operator::Minus => match (left_type, right_type) {
-                        // T - T != T if T is a datetime / date
-                        (Datetime(tul, _), Datetime(tur, _)) => {
-                            Duration(get_time_units(&tul, &tur))
+                    _ => {
+                        // don't traverse tree until strictly needed. Can have terrible performance.
+                        // # 3210
+                        let left_type = arena.get(*left).get_type(schema, ctxt, arena)?;
+                        let right_type = arena.get(*right).get_type(schema, ctxt, arena)?;
+
+                        match op {
+                            Operator::Minus => match (left_type, right_type) {
+                                // T - T != T if T is a datetime / date
+                                (Datetime(tul, _), Datetime(tur, _)) => {
+                                    Duration(get_time_units(&tul, &tur))
+                                }
+                                (Date, Date) => Duration(TimeUnit::Milliseconds),
+                                (left, right) => get_supertype(&left, &right)?,
+                            },
+                            _ => get_supertype(&left_type, &right_type)?,
                         }
-                        (Date, Date) => Duration(TimeUnit::Milliseconds),
-                        (left, right) => get_supertype(&left, &right)?,
-                    },
-                    _ => get_supertype(&left_type, &right_type)?,
+                    }
                 };
 
                 let out_field;
@@ -203,7 +257,8 @@ impl AExpr {
                 use AAggExpr::*;
                 match agg {
                     Max(expr) | Sum(expr) | Min(expr) | First(expr) | Last(expr) => {
-                        arena.get(*expr).to_field(schema, ctxt, arena)
+                        // default context because `col()` would return a list in aggregation context
+                        arena.get(*expr).to_field(schema, Context::Default, arena)
                     }
                     Median(expr) => {
                         let mut field = arena.get(*expr).to_field(schema, ctxt, arena)?;
@@ -218,7 +273,9 @@ impl AExpr {
                         Ok(field)
                     }
                     List(expr) => {
-                        let mut field = arena.get(*expr).to_field(schema, ctxt, arena)?;
+                        // default context because `col()` would return a list in aggregation context
+                        let mut field =
+                            arena.get(*expr).to_field(schema, Context::Default, arena)?;
                         field.coerce(DataType::List(field.data_type().clone().into()));
                         Ok(field)
                     }
@@ -272,7 +329,7 @@ impl AExpr {
                     Ok(truthy)
                 }
             }
-            Function {
+            AnonymousFunction {
                 output_type, input, ..
             } => {
                 let fields = input
@@ -281,6 +338,15 @@ impl AExpr {
                     .collect::<Result<Vec<_>>>()?;
                 Ok(output_type.get_field(schema, ctxt, &fields))
             }
+            Function {
+                function, input, ..
+            } => {
+                let fields = input
+                    .iter()
+                    .map(|node| arena.get(*node).to_field(schema, ctxt, arena))
+                    .collect::<Result<Vec<_>>>()?;
+                function.get_field(schema, ctxt, &fields)
+            }
             Shift { input, .. } => arena.get(*input).to_field(schema, ctxt, arena),
             Slice { input, .. } => arena.get(*input).to_field(schema, ctxt, arena),
             Wildcard => panic!("should be no wildcard at this point"),
