@@ -57,6 +57,7 @@ macro_rules! push_expr {
                 $push(falsy);
                 $push(predicate)
             }
+            AnonymousFunction { input, .. } => input.$iter().for_each(|e| $push(e)),
             Function { input, .. } => input.$iter().for_each(|e| $push(e)),
             Shift { input, .. } => $push(input),
             Reverse(e) => $push(e),
@@ -77,7 +78,15 @@ macro_rules! push_expr {
                     $push(e);
                 }
             }
-            Slice { input, .. } => $push(input),
+            Slice {
+                input,
+                offset,
+                length,
+            } => {
+                $push(input);
+                $push(offset);
+                $push(length);
+            }
             Exclude(e, _) => $push(e),
             KeepName(e) => $push(e),
             RenameAlias { expr, .. } => $push(expr),
@@ -208,7 +217,7 @@ impl AExpr {
                 push(falsy);
                 push(predicate)
             }
-            Function { input, .. } => input.iter().for_each(push),
+            AnonymousFunction { input, .. } | Function { input, .. } => input.iter().for_each(push),
             Shift { input, .. } => push(input),
             Reverse(e) => push(e),
             Duplicated(e) => push(e),
@@ -228,7 +237,15 @@ impl AExpr {
                     push(e);
                 }
             }
-            Slice { input, .. } => push(input),
+            Slice {
+                input,
+                offset,
+                length,
+            } => {
+                push(input);
+                push(offset);
+                push(length);
+            }
         }
     }
 }
@@ -311,7 +328,7 @@ mod test {
 
         let (root, lp_arena, _expr_arena) = df
             .lazy()
-            .sort("a", false)
+            .sort("a", Default::default())
             .groupby([col("a")])
             .agg([col("a").first()])
             .logical_plan
