@@ -83,7 +83,7 @@ impl OptimizationRule for AggregatePushdown {
                 input,
                 schema,
             } => self.pushdown_projection(node, expr, input, schema, lp_arena, expr_arena),
-            // todo! hstack should pushown not dependent columns
+            // todo! hstack should pushdown not dependent columns
             Join { .. } | Aggregate { .. } | HStack { .. } | DataFrameScan { .. } => {
                 if self.accumulated_projections.is_empty() {
                     lp_arena.replace(node, lp);
@@ -94,7 +94,7 @@ impl OptimizationRule for AggregatePushdown {
                     let input_schema = lp_arena.get(new_node).schema(lp_arena);
 
                     let nodes: Vec<_> = self.process_nodes();
-                    let fields = self
+                    let fields: Vec<_> = self
                         .accumulated_projections
                         .iter()
                         .map(|n| {
@@ -108,7 +108,7 @@ impl OptimizationRule for AggregatePushdown {
                     Some(ALogicalPlan::Projection {
                         expr: nodes,
                         input: new_node,
-                        schema: Arc::new(Schema::new(fields)),
+                        schema: Arc::new(Schema::from(fields)),
                     })
                 }
             }
@@ -152,11 +152,9 @@ impl OptimizationRule for AggregatePushdown {
                 path,
                 schema,
                 output_schema,
-                with_columns,
                 predicate,
                 aggregate,
-                n_rows,
-                cache,
+                options,
             } => match self.accumulated_projections.is_empty() {
                 true => {
                     lp_arena.replace(
@@ -165,11 +163,9 @@ impl OptimizationRule for AggregatePushdown {
                             path,
                             schema,
                             output_schema,
-                            with_columns,
                             predicate,
                             aggregate,
-                            n_rows,
-                            cache,
+                            options,
                         },
                     );
                     None
@@ -180,11 +176,9 @@ impl OptimizationRule for AggregatePushdown {
                         path,
                         schema,
                         output_schema,
-                        with_columns,
                         predicate,
                         aggregate,
-                        n_rows,
-                        cache,
+                        options,
                     })
                 }
             },
