@@ -2,11 +2,11 @@ use crate::prelude::*;
 use polars_core::{datatypes::PlHashMap, prelude::*};
 
 pub(crate) mod aggregate_pushdown;
-#[cfg(any(feature = "parquet", feature = "csv-file"))]
-pub(crate) mod aggregate_scan_projections;
 pub(crate) mod delay_rechunk;
 pub(crate) mod drop_nulls;
 pub(crate) mod fast_projection;
+#[cfg(any(feature = "ipc", feature = "parquet", feature = "csv-file"))]
+pub(crate) mod file_caching;
 pub(crate) mod predicate_pushdown;
 pub(crate) mod projection_pushdown;
 pub(crate) mod simplify_expr;
@@ -26,6 +26,6 @@ pub trait Optimize {
 // arbitrary constant to reduce reallocation.
 const HASHMAP_SIZE: usize = 16;
 
-pub(crate) fn init_hashmap<K, V>() -> PlHashMap<K, V> {
-    PlHashMap::with_capacity(HASHMAP_SIZE)
+pub(crate) fn init_hashmap<K, V>(max_len: Option<usize>) -> PlHashMap<K, V> {
+    PlHashMap::with_capacity(std::cmp::min(max_len.unwrap_or(HASHMAP_SIZE), HASHMAP_SIZE))
 }
