@@ -1,4 +1,4 @@
-use crate::physical_plan::state::ExecutionState;
+use crate::physical_plan::state::{ExecutionState, StateFlags};
 use crate::prelude::*;
 use polars_core::frame::groupby::GroupsProxy;
 use polars_core::prelude::*;
@@ -87,15 +87,26 @@ impl PhysicalExpr for TernaryExpr {
         &self.expr
     }
     fn evaluate(&self, df: &DataFrame, state: &ExecutionState) -> Result<Series> {
-        let mask_series = self.predicate.evaluate(df, state)?;
+        let mut state = state.split();
+        // don't cache window functions as they run in parallel
+        state.flags.remove(StateFlags::CACHE_WINDOW_EXPR);
+        let mask_series = self.predicate.evaluate(df, &state)?;
         let mut mask = mask_series.bool()?.clone();
 
-        let op_truthy = || self.truthy.evaluate(df, state);
-        let op_falsy = || self.falsy.evaluate(df, state);
+        let op_truthy = || self.truthy.evaluate(df, &state);
+        let op_falsy = || self.falsy.evaluate(df, &state);
 
         let (truthy, falsy) = POOL.install(|| rayon::join(op_truthy, op_falsy));
         let mut truthy = truthy?;
         let mut falsy = falsy?;
+
+        if truthy.is_empty() {
+            return Ok(truthy);
+        }
+        if falsy.is_empty() {
+            return Ok(falsy);
+        }
+
         expand_lengths(&mut truthy, &mut falsy, &mut mask);
 
         truthy.zip_with(&mask, &falsy)
@@ -111,8 +122,6 @@ impl PhysicalExpr for TernaryExpr {
         groups: &'a GroupsProxy,
         state: &ExecutionState,
     ) -> Result<AggregationContext<'a>> {
-        let required_height = df.height();
-
         let op_mask = || self.predicate.evaluate_on_groups(df, groups, state);
         let op_truthy = || self.truthy.evaluate_on_groups(df, groups, state);
         let op_falsy = || self.falsy.evaluate_on_groups(df, groups, state);
@@ -130,50 +139,64 @@ impl PhysicalExpr for TernaryExpr {
             "cannot combine this ternary expression, the groups do not match"
         );
 
+        use AggState::*;
         match (ac_truthy.agg_state(), ac_falsy.agg_state()) {
-            // if the groups_len == df.len we can just apply all flat.
-            (AggState::AggregatedFlat(s), AggState::NotAggregated(_) | AggState::Literal(_))
-                if s.len() != df.height() =>
-            {
-                finish_as_iters(ac_truthy, ac_falsy, ac_mask)
-            }
-            // all aggregated or literal
+            // all branches are aggregated-flat or literal
+            // mask -> aggregated-flat
+            // truthy -> aggregated-flat | literal
+            // falsy -> aggregated-flat | literal
             // simply align lengths and zip
             (
-                AggState::Literal(truthy) | AggState::AggregatedFlat(truthy),
-                AggState::AggregatedFlat(falsy) | AggState::Literal(falsy),
+                Literal(truthy) | AggregatedFlat(truthy),
+                AggregatedFlat(falsy) | Literal(falsy),
             )
-            | (AggState::AggregatedList(truthy), AggState::AggregatedList(falsy))
-                if matches!(ac_mask.agg_state(), AggState::AggregatedFlat(_)) =>
-            {
-                let mut truthy = truthy.clone();
-                let mut falsy = falsy.clone();
-                let mut mask = ac_mask.series().bool()?.clone();
-                expand_lengths(&mut truthy, &mut falsy, &mut mask);
-                let mut out = truthy.zip_with(&mask, &falsy).unwrap();
-                out.rename(truthy.name());
-                ac_truthy.with_series(out, true);
-                Ok(ac_truthy)
-            }
+            | (AggregatedList(truthy), AggregatedList(falsy))
+            if matches!(ac_mask.agg_state(), AggState::AggregatedFlat(_))
+            =>
+                {
+                    let mut truthy = truthy.clone();
+                    let mut falsy = falsy.clone();
+                    let mut mask = ac_mask.series().bool()?.clone();
+                    expand_lengths(&mut truthy, &mut falsy, &mut mask);
+                    let mut out = truthy.zip_with(&mask, &falsy).unwrap();
+                    out.rename(truthy.name());
+                    ac_truthy.with_series(out, true);
+                    Ok(ac_truthy)
+                }
+
+            // Fallthrough from previous branch. That means the mask is not aggregated
+            // and may be longer than branches, so we iterate over the groups
+            (AggregatedFlat(agg), Literal(_)) |
+            (Literal(_), AggregatedFlat(agg))
+            // todo! check if we need to take this branch for those
+            //| (AggregatedFlat(agg), NotAggregated(_)) |
+            // (NotAggregated(_), AggregatedFlat(agg))
+
             // if the groups_len == df.len we can just apply all flat.
-            (AggState::NotAggregated(_) | AggState::Literal(_), AggState::AggregatedFlat(s))
-                if s.len() != df.height() =>
-            {
+            // make sure that the order of the aggregation does not matter!
+            // this is the case when we have a flat aggregation combined with a literal:
+            // `sum(..) - lit(..)`.
+            if agg.len() != df.height()
+            => {
+                finish_as_iters(ac_truthy, ac_falsy, ac_mask)
+            }
+
+            // Same branch as above, but without escape hatch for `num_groups == df.height()`
+
+            // we cannot flatten a list because that changes the order, so we apply over groups
+            (AggregatedList(_), NotAggregated(_)) |
+            (NotAggregated(_), AggregatedList(_)) => {
                 finish_as_iters(ac_truthy, ac_falsy, ac_mask)
             }
 
             // Both are or a flat series or aggregated into a list
             // so we can flatten the Series an apply the operators
             _ => {
-                let mask = mask_s.bool()?;
-                let out = ac_truthy
-                    .flat_naive()
-                    .zip_with(mask, ac_falsy.flat_naive().as_ref())?;
-
-                assert!((out.len() == required_height), "The output of the `when -> then -> otherwise-expr` is of a different length than the groups.\
-The expr produced {} values. Where the original DataFrame has {} values",
-                        out.len(),
-                        required_height);
+                let mut mask = mask_s.bool()?.clone();
+                let mut truthy = ac_truthy.flat_naive().into_owned();
+                let mut falsy = ac_falsy.flat_naive().into_owned();
+                expand_lengths(&mut truthy, &mut falsy, &mut mask);
+                let out = truthy.zip_with(&mask, &falsy)?;
 
                 ac_truthy.with_series(out, false);
 
