@@ -1,5 +1,7 @@
 pub mod executors;
 pub mod expressions;
+#[cfg(any(feature = "ipc", feature = "parquet", feature = "csv-file"))]
+mod file_cache;
 pub mod planner;
 pub(crate) mod state;
 
@@ -29,12 +31,12 @@ pub trait PhysicalPlanner {
 /// Executors have other executors as input. By having a tree of executors we can execute the
 /// physical plan until the last executor is evaluated.
 pub trait Executor: Send + Sync {
-    fn execute(&mut self, cache: &ExecutionState) -> Result<DataFrame>;
+    fn execute(&mut self, cache: &mut ExecutionState) -> Result<DataFrame>;
 }
 
 pub struct Dummy {}
 impl Executor for Dummy {
-    fn execute(&mut self, _cache: &ExecutionState) -> Result<DataFrame> {
+    fn execute(&mut self, _cache: &mut ExecutionState) -> Result<DataFrame> {
         panic!("should not get here");
     }
 }
