@@ -1,3 +1,4 @@
+use crate::IdxSize;
 use rayon::{prelude::*, ThreadPool};
 
 /// This is a perfect sort particularly useful for an argsort of an argsort
@@ -12,19 +13,19 @@ use rayon::{prelude::*, ThreadPool};
 /// - The left indices are placed at the location right points to.
 ///
 /// # Safety
-/// The caller must ensure that the right indexes fo `&[(_, u32)]` are integers ranging from `0..idx.len`
-pub unsafe fn perfect_sort(pool: &ThreadPool, idx: &[(u32, u32)]) -> Vec<u32> {
+/// The caller must ensure that the right indexes fo `&[(_, IdxSize)]` are integers ranging from `0..idx.len`
+pub unsafe fn perfect_sort(pool: &ThreadPool, idx: &[(IdxSize, IdxSize)]) -> Vec<IdxSize> {
     let chunk_size = std::cmp::max(
         idx.len() / pool.current_num_threads(),
         pool.current_num_threads(),
     );
 
-    let mut out: Vec<u32> = Vec::with_capacity(idx.len());
-    let ptr = out.as_mut_ptr() as *const u32 as usize;
+    let mut out: Vec<IdxSize> = Vec::with_capacity(idx.len());
+    let ptr = out.as_mut_ptr() as *const IdxSize as usize;
 
     pool.install(|| {
         idx.par_chunks(chunk_size).for_each(|indices| {
-            let ptr = ptr as *mut u32;
+            let ptr = ptr as *mut IdxSize;
             for (idx_val, idx_location) in indices {
                 // Safety:
                 // idx_location is in bounds by invariant of this function
