@@ -16,7 +16,14 @@ from polars.cfg import (  # flake8: noqa. We do not export in __all__
     Config,
     toggle_string_cache,
 )
-from polars.convert import from_arrow, from_dict, from_dicts, from_pandas, from_records
+from polars.convert import (
+    from_arrow,
+    from_dict,
+    from_dicts,
+    from_numpy,
+    from_pandas,
+    from_records,
+)
 from polars.datatypes import (
     Boolean,
     Categorical,
@@ -58,7 +65,7 @@ from polars.internals.frame import (  # flake8: noqa # TODO: remove need for wra
     DataFrame,
     wrap_df,
 )
-from polars.internals.functions import arg_where, concat, date_range, get_dummies
+from polars.internals.functions import concat, date_range, get_dummies
 from polars.internals.io import read_ipc_schema, read_parquet_schema
 from polars.internals.lazy_frame import LazyFrame
 from polars.internals.lazy_functions import _date as date
@@ -68,6 +75,7 @@ from polars.internals.lazy_functions import (
     any,
     apply,
     arange,
+    arg_where,
     argsort_by,
     avg,
     col,
@@ -230,6 +238,7 @@ __all__ = [
     "from_dict",
     "from_dicts",
     "from_records",
+    "from_numpy",
     "from_arrow",
     "from_pandas",
     # testing
