@@ -4,7 +4,9 @@ use crate::dataframe::PyDataFrame;
 use crate::error::PyPolarsErr;
 use crate::file::get_file_like;
 use crate::lazy::{dsl::PyExpr, utils::py_exprs_to_exprs};
-use crate::prelude::{IdxSize, LogicalPlan, NullValues, ScanArgsIpc, ScanArgsParquet};
+use crate::prelude::{
+    IdxSize, LogicalPlan, NullValues, ParallelStrategy, ScanArgsIpc, ScanArgsParquet,
+};
 use polars::io::RowCount;
 use polars::lazy::frame::{AllowedOptimizations, LazyCsvReader, LazyFrame, LazyGroupBy};
 use polars::lazy::prelude::col;
@@ -222,7 +224,7 @@ impl PyLazyFrame {
         path: String,
         n_rows: Option<usize>,
         cache: bool,
-        parallel: bool,
+        parallel: Wrap<ParallelStrategy>,
         rechunk: bool,
         row_count: Option<(String, IdxSize)>,
     ) -> PyResult<Self> {
@@ -230,7 +232,7 @@ impl PyLazyFrame {
         let args = ScanArgsParquet {
             n_rows,
             cache,
-            parallel,
+            parallel: parallel.0,
             rechunk,
             row_count,
         };
@@ -323,10 +325,15 @@ impl PyLazyFrame {
         .into()
     }
 
-    pub fn sort_by_exprs(&self, by_column: Vec<PyExpr>, reverse: Vec<bool>) -> PyLazyFrame {
+    pub fn sort_by_exprs(
+        &self,
+        by_column: Vec<PyExpr>,
+        reverse: Vec<bool>,
+        nulls_last: bool,
+    ) -> PyLazyFrame {
         let ldf = self.ldf.clone();
         let exprs = py_exprs_to_exprs(by_column);
-        ldf.sort_by_exprs(exprs, reverse).into()
+        ldf.sort_by_exprs(exprs, reverse, nulls_last).into()
     }
     pub fn cache(&self) -> PyLazyFrame {
         let ldf = self.ldf.clone();
@@ -652,9 +659,9 @@ impl PyLazyFrame {
             .into()
     }
 
-    pub fn slice(&self, offset: i64, len: IdxSize) -> Self {
+    pub fn slice(&self, offset: i64, len: Option<IdxSize>) -> Self {
         let ldf = self.ldf.clone();
-        ldf.slice(offset, len).into()
+        ldf.slice(offset, len.unwrap_or(IdxSize::MAX)).into()
     }
 
     pub fn tail(&self, n: IdxSize) -> Self {
