@@ -1,9 +1,8 @@
 #![feature(vec_into_raw_parts)]
 #![allow(clippy::nonstandard_macro_braces)] // needed because clippy does not understand proc macro of pyo3
 #![allow(clippy::transmute_undefined_repr)]
-#[macro_use]
-extern crate polars;
 extern crate core;
+extern crate polars;
 
 use pyo3::prelude::*;
 use pyo3::wrap_pyfunction;
@@ -470,6 +469,11 @@ fn pool_size() -> usize {
     POOL.current_num_threads()
 }
 
+#[pyfunction]
+pub fn arg_where(condition: PyExpr) -> PyExpr {
+    polars::lazy::dsl::arg_where(condition.inner).into()
+}
+
 #[pymodule]
 fn polars(py: Python, m: &PyModule) -> PyResult<()> {
     m.add("NotFoundError", py.get_type::<NotFoundError>())
@@ -530,5 +534,6 @@ fn polars(py: Python, m: &PyModule) -> PyResult<()> {
     m.add_wrapped(wrap_pyfunction!(as_struct)).unwrap();
     m.add_wrapped(wrap_pyfunction!(repeat)).unwrap();
     m.add_wrapped(wrap_pyfunction!(pool_size)).unwrap();
+    m.add_wrapped(wrap_pyfunction!(arg_where)).unwrap();
     Ok(())
 }
