@@ -1,4 +1,6 @@
 # flake8: noqa: W191,E101
+from __future__ import annotations
+
 from os import path
 from pathlib import Path
 
@@ -57,7 +59,7 @@ def test_categorical_parquet_statistics(io_test_dir: str) -> None:
         .write_parquet(file, statistics=True)
     )
 
-    for par in [True, False]:
+    for par in ["auto", "columns", "row_groups", "none"]:
         df = (
             pl.scan_parquet(file, parallel=par)
             .filter(pl.col("book") == "bookA")
