@@ -1,8 +1,10 @@
 # flake8: noqa: W191,E101
+from __future__ import annotations
+
 import sys
 import typing
 from builtins import range
-from datetime import date, datetime, time
+from datetime import date, datetime, timedelta
 from io import BytesIO
 from typing import Any, Iterator, Type
 from unittest.mock import patch
@@ -13,7 +15,7 @@ import pyarrow as pa
 import pytest
 
 import polars as pl
-from polars import testing
+from polars.testing import assert_frame_equal, assert_series_equal, columns
 
 if sys.version_info >= (3, 8):
     from typing import Literal
@@ -22,13 +24,19 @@ else:
 
 
 def test_version() -> None:
-    pl.__version__
+    _version = pl.__version__
+
+
+def test_null_count() -> None:
+    df = pl.DataFrame({"a": [2, 1, 3], "b": ["a", "b", None]})
+    assert df.null_count().shape == (1, 2)
 
 
 def test_init_empty() -> None:
-    # Empty initialization
-    df1 = pl.DataFrame()
-    assert df1.shape == (0, 0)
+    # test various flavours of empty init
+    for empty in (None, (), [], {}, pa.Table.from_arrays([])):
+        df = pl.DataFrame(empty)
+        assert df.shape == (0, 0)
 
 
 def test_init_only_columns() -> None:
@@ -64,6 +72,20 @@ def test_init_only_columns() -> None:
         assert df.dtypes == [pl.Date, pl.UInt64, pl.Int8, pl.List]
         assert getattr(df.schema["d"], "inner") == pl.UInt8
 
+        dfe = df.cleared()
+        assert (df.schema == dfe.schema) and (dfe.shape == df.shape)
+
+
+def test_special_char_colname_init() -> None:
+    from string import punctuation
+
+    cols = [(c.name, c.dtype) for c in columns(punctuation)]
+    df = pl.DataFrame(columns=cols)
+
+    assert len(cols) == len(df.columns)
+    assert 0 == len(df.rows())
+    assert df.is_empty()
+
 
 def test_init_dict() -> None:
     # Empty dictionary
@@ -141,6 +163,9 @@ def test_init_dict() -> None:
     )
     assert df.schema == {"c": pl.Int8, "d": pl.Int16}
 
+    dfe = df.cleared()
+    assert (df.schema == dfe.schema) and (len(dfe) == 0)
+
 
 def test_init_ndarray() -> None:
     # Empty array
@@ -205,6 +230,11 @@ def test_init_ndarray() -> None:
     with pytest.raises(ValueError):
         _ = pl.DataFrame(np.random.randn(2, 2, 2))
 
+    # numpy not available
+    with patch("polars.internals.frame._NUMPY_AVAILABLE", False):
+        with pytest.raises(ValueError):
+            pl.DataFrame(np.array([1, 2, 3]), columns=["a"])
+
 
 # TODO: Remove this test case when removing deprecated behaviour
 def test_init_ndarray_deprecated() -> None:
@@ -510,15 +540,74 @@ def test_assignment() -> None:
     assert df["foo"].to_list() == [1, 9, 9]
 
 
+def test_select_at_idx() -> None:
+    df = pl.DataFrame({"x": [1, 2, 3], "y": [2, 3, 4], "z": [3, 4, 5]})
+    for idx in range(len(df.columns)):
+        assert_series_equal(
+            df.select_at_idx(idx),  # regular positive indexing
+            df.select_at_idx(idx - len(df.columns)),  # equivalent negative index
+        )
+
+
+def test_insert_at_idx() -> None:
+    df = pl.DataFrame({"z": [3, 4, 5]})
+    df.insert_at_idx(0, pl.Series("x", [1, 2, 3]))
+    df.insert_at_idx(-1, pl.Series("y", [2, 3, 4]))
+
+    expected_df = pl.DataFrame({"x": [1, 2, 3], "y": [2, 3, 4], "z": [3, 4, 5]})
+    assert_frame_equal(expected_df, df)
+
+
+def test_replace_at_idx() -> None:
+    df = pl.DataFrame({"x": [1, 2, 3], "y": [2, 3, 4], "z": [3, 4, 5]})
+    df.replace_at_idx(0, pl.Series("a", [4, 5, 6]))
+    df.replace_at_idx(-2, pl.Series("b", [5, 6, 7]))
+    df.replace_at_idx(-1, pl.Series("c", [6, 7, 8]))
+
+    expected_df = pl.DataFrame({"a": [4, 5, 6], "b": [5, 6, 7], "c": [6, 7, 8]})
+    assert_frame_equal(expected_df, df)
+
+
+def test_to_series() -> None:
+    df = pl.DataFrame({"x": [1, 2, 3], "y": [2, 3, 4], "z": [3, 4, 5]})
+
+    assert_series_equal(df.to_series(), df["x"])
+    assert_series_equal(df.to_series(0), df["x"])
+    assert_series_equal(df.to_series(-3), df["x"])
+
+    assert_series_equal(df.to_series(1), df["y"])
+    assert_series_equal(df.to_series(-2), df["y"])
+
+    assert_series_equal(df.to_series(2), df["z"])
+    assert_series_equal(df.to_series(-1), df["z"])
+
+
+def test_take_every() -> None:
+    df = pl.DataFrame({"a": [1, 2, 3, 4], "b": ["w", "x", "y", "z"]})
+    expected_df = pl.DataFrame({"a": [1, 3], "b": ["w", "y"]})
+    assert_frame_equal(expected_df, df.take_every(2))
+
+
 def test_slice() -> None:
-    df = pl.DataFrame({"a": [2, 1, 3], "b": ["a", "b", "c"]})
-    df = df.slice(1, 2)
-    assert df.frame_equal(pl.DataFrame({"a": [1, 3], "b": ["b", "c"]}))
+    df = pl.DataFrame({"a": [1, 2, 3], "b": ["a", "b", "c"]})
+    expected = pl.DataFrame({"a": [2, 3], "b": ["b", "c"]})
+    for slice_params in (
+        [1, 10],  # slice > len(df)
+        [1, 2],  # slice == len(df)
+        [1],  # optional len
+    ):
+        assert df.slice(*slice_params).frame_equal(expected)
 
-
-def test_null_count() -> None:
-    df = pl.DataFrame({"a": [2, 1, 3], "b": ["a", "b", None]})
-    assert df.null_count().shape == (1, 2)
+    for py_slice in (
+        slice(1, 2),
+        slice(0, 2, 2),
+        slice(3, -3, -1),
+        slice(1, None, -2),
+        slice(-1, -3, -1),
+        slice(-3, None, -3),
+    ):
+        # confirm frame slice matches python slice
+        assert df[py_slice].rows() == df.rows()[py_slice]
 
 
 def test_head_tail_limit() -> None:
@@ -599,8 +688,9 @@ def test_groupby() -> None:
     # )
     assert df.groupby("a").apply(lambda df: df[["c"]].sum()).sort("c")["c"][0] == 1
 
-    df_groups = df.groupby("a").groups().sort("a")
-    assert df_groups["a"].series_equal(pl.Series("a", ["a", "b", "c"]))
+    with pytest.deprecated_call():
+        df_groups = df.groupby("a").groups().sort("a")
+        assert df_groups["a"].series_equal(pl.Series("a", ["a", "b", "c"]))
 
     with pytest.deprecated_call():
         # TODO: find a way to avoid indexing into GroupBy
@@ -609,9 +699,10 @@ def test_groupby() -> None:
             if subdf["a"][0] == "b":
                 assert subdf.shape == (3, 3)
 
-    assert df.groupby("a").get_group("c").shape == (1, 3)
-    assert df.groupby("a").get_group("b").shape == (3, 3)
-    assert df.groupby("a").get_group("a").shape == (2, 3)
+    with pytest.deprecated_call():
+        assert df.groupby("a").get_group("c").shape == (1, 3)
+        assert df.groupby("a").get_group("b").shape == (3, 3)
+        assert df.groupby("a").get_group("a").shape == (2, 3)
 
     # Use lazy API in eager groupby
     assert df.groupby("a").agg([pl.sum("b")]).shape == (3, 2)
@@ -1042,7 +1133,7 @@ def test_concat() -> None:
 
 def test_arg_where() -> None:
     s = pl.Series([True, False, True, False])
-    assert pl.arg_where(s).cast(int).series_equal(pl.Series([0, 2]))
+    assert pl.arg_where(s, eager=True).cast(int).series_equal(pl.Series([0, 2]))
 
 
 def test_get_dummies() -> None:
@@ -1181,9 +1272,6 @@ def test_lazy_functions() -> None:
     expected = 3
     assert np.isclose(out.select_at_idx(9), expected)
     assert np.isclose(pl.last(df["b"]), expected)
-    expected = 3
-    assert np.isclose(out.select_at_idx(9), expected)
-    assert np.isclose(pl.last(df["b"]), expected)
 
 
 def test_multiple_column_sort() -> None:
@@ -1289,6 +1377,7 @@ def test_to_html(df: pl.DataFrame) -> None:
 def test_rows() -> None:
     df = pl.DataFrame({"a": [1, 2], "b": [1, 2]})
     assert df.rows() == [(1, 1), (2, 2)]
+    assert df.reverse().rows() == [(2, 2), (1, 1)]
 
 
 def test_rename(df: pl.DataFrame) -> None:
@@ -1344,12 +1433,14 @@ def test_repeat_by() -> None:
 
 
 def test_join_dates() -> None:
-    date_times = pd.date_range(
-        "2021-06-24 00:00:00", "2021-06-24 10:00:00", freq="1H", closed="left"
+    dts_in = pl.date_range(
+        datetime(2021, 6, 24),
+        datetime(2021, 6, 24, 10, 0, 0),
+        interval=timedelta(hours=1),
+        closed="left",
     )
     dts = (
-        pl.from_pandas(date_times)
-        .cast(int)
+        dts_in.cast(int)
         .apply(lambda x: x + np.random.randint(1_000 * 60, 60_000 * 60))
         .cast(pl.Datetime)
     )
@@ -1362,7 +1453,8 @@ def test_join_dates() -> None:
             "value": [2, 3, 4, 1, 2, 3, 5, 1, 2, 3],
         }
     )
-    df.join(df, on="datetime")
+    out = df.join(df, on="datetime")
+    assert len(out) == len(df)
 
 
 def test_asof_cross_join() -> None:
@@ -1474,13 +1566,13 @@ def test_panic() -> None:
 def test_h_agg() -> None:
     df = pl.DataFrame({"a": [1, None, 3], "b": [1, 2, 3]})
 
-    pl.testing.assert_series_equal(
+    assert_series_equal(
         df.sum(axis=1, null_strategy="ignore"), pl.Series("a", [2, 2, 6])
     )
-    pl.testing.assert_series_equal(
+    assert_series_equal(
         df.sum(axis=1, null_strategy="propagate"), pl.Series("a", [2, None, 6])
     )
-    pl.testing.assert_series_equal(
+    assert_series_equal(
         df.mean(axis=1, null_strategy="propagate"), pl.Series("a", [1.0, None, 3.0])
     )
 
@@ -1905,7 +1997,7 @@ def test_add_string() -> None:
 def test_getattr() -> None:
     with pytest.deprecated_call():
         df = pl.DataFrame({"a": [1.0, 2.0]})
-        testing.assert_series_equal(df.a, pl.Series("a", [1.0, 2.0]))
+        assert_series_equal(df.a, pl.Series("a", [1.0, 2.0]))
 
         with pytest.raises(AttributeError):
             _ = df.b
@@ -1918,9 +2010,6 @@ def test_get_item() -> None:
     # expression
     assert df.select(pl.col("a")).frame_equal(pl.DataFrame({"a": [1.0, 2.0]}))
 
-    # numpy array
-    assert df[np.array([True, False])].frame_equal(pl.DataFrame({"a": [1.0], "b": [3]}))
-
     # tuple. The first element refers to the rows, the second element to columns
     assert df[:, :].frame_equal(df)
 
@@ -1937,12 +2026,16 @@ def test_get_item() -> None:
     assert df[::2].frame_equal(pl.DataFrame({"a": [1.0], "b": [3]}))
 
     # numpy array; assumed to be row indices if integers, or columns if strings
-    # TODO: add boolean mask support
     df[np.array([1])].frame_equal(pl.DataFrame({"a": [2.0], "b": [4]}))
     df[np.array(["a"])].frame_equal(pl.DataFrame({"a": [1.0, 2.0]}))
     # note that we cannot use floats (even if they could be casted to integer without loss)
     with pytest.raises(NotImplementedError):
         _ = df[np.array([1.0])]
+    # using boolean masks with numpy is deprecated
+    with pytest.deprecated_call():
+        assert df[np.array([True, False])].frame_equal(
+            pl.DataFrame({"a": [1.0], "b": [3]})
+        )
 
     # sequences (lists or tuples; tuple only if length != 2)
     # if strings or list of expressions, assumed to be column names
@@ -2051,10 +2144,10 @@ def test_join_suffixes() -> None:
     df_b = pl.DataFrame({"A": [1], "B": [1]})
 
     for how in ["left", "inner", "outer", "cross"]:
-        # no need for an essert, we error if wrong
-        df_a.join(df_b, on="A", suffix="_y", how=how)["B_y"]
+        # no need for an assert, we error if wrong
+        _df = df_a.join(df_b, on="A", suffix="_y", how=how)["B_y"]
 
-    df_a.join_asof(df_b, on="A", suffix="_y")["B_y"]
+    _df = df_a.join_asof(df_b, on="A", suffix="_y")["B_y"]
 
 
 def test_preservation_of_subclasses() -> None:
@@ -2081,7 +2174,7 @@ def test_preservation_of_subclasses() -> None:
     # and connect these classes together
     class MyLazyFrame(pl.LazyFrame):
         @property
-        def _dataframe_class(cls) -> "Type[MyDataFrame]":
+        def _dataframe_class(cls) -> type[MyDataFrame]:
             return MyDataFrame
 
     class MyDataFrame(pl.DataFrame):
@@ -2236,3 +2329,105 @@ def test_fill_null_limits() -> None:
             False,
         ],
     }
+
+
+def test_selection_regex_and_multicol() -> None:
+    test_df = pl.DataFrame(
+        {
+            "a": [1, 2, 3, 4],
+            "b": [5, 6, 7, 8],
+            "c": [9, 10, 11, 12],
+            "foo": [13, 14, 15, 16],
+        }
+    )
+
+    # Selection only
+    test_df.select(
+        [
+            pl.col(["a", "b", "c"]).suffix("_list"),
+            pl.all().exclude("foo").suffix("_wild"),
+            pl.col("^\\w$").suffix("_regex"),
+        ]
+    )
+
+    # Multi * Single
+    assert test_df.select(pl.col(["a", "b", "c"]) * pl.col("foo")).to_dict(False) == {
+        "a": [13, 28, 45, 64],
+        "b": [65, 84, 105, 128],
+        "c": [117, 140, 165, 192],
+    }
+    assert test_df.select(pl.all().exclude("foo") * pl.col("foo")).to_dict(False) == {
+        "a": [13, 28, 45, 64],
+        "b": [65, 84, 105, 128],
+        "c": [117, 140, 165, 192],
+    }
+
+    assert test_df.select(pl.col("^\\w$") * pl.col("foo")).to_dict(False) == {
+        "a": [13, 28, 45, 64],
+        "b": [65, 84, 105, 128],
+        "c": [117, 140, 165, 192],
+    }
+
+    # Multi * Multi
+    assert test_df.select(pl.col(["a", "b", "c"]) * pl.col(["a", "b", "c"])).to_dict(
+        False
+    ) == {"a": [1, 4, 9, 16], "b": [25, 36, 49, 64], "c": [81, 100, 121, 144]}
+    assert test_df.select(pl.all().exclude("foo") * pl.all().exclude("foo")).to_dict(
+        False
+    ) == {"a": [1, 4, 9, 16], "b": [25, 36, 49, 64], "c": [81, 100, 121, 144]}
+    assert test_df.select(pl.col("^\\w$") * pl.col("^\\w$")).to_dict(False) == {
+        "a": [1, 4, 9, 16],
+        "b": [25, 36, 49, 64],
+        "c": [81, 100, 121, 144],
+    }
+
+
+def test_with_columns() -> None:
+    df = pl.DataFrame(
+        {
+            "a": [1, 2, 3, 4],
+            "b": [0.5, 4, 10, 13],
+            "c": [True, True, False, True],
+        }
+    )
+    srs_named = pl.Series("f", [3, 2, 1, 0])
+    srs_unnamed = pl.Series(values=[3, 2, 1, 0])
+
+    expected = pl.DataFrame(
+        {
+            "a": [1, 2, 3, 4],
+            "b": [0.5, 4, 10, 13],
+            "c": [True, True, False, True],
+            "d": [0.5, 8.0, 30.0, 52.0],
+            "e": [False, False, True, False],
+            "f": [3, 2, 1, 0],
+        }
+    )
+
+    # as exprs list
+    dx = df.with_columns(
+        [(pl.col("a") * pl.col("b")).alias("d"), ~pl.col("c").alias("e"), srs_named]
+    )
+    assert_frame_equal(dx, expected)
+
+    # as **kwargs (experimental feature: requires opt-in)
+    pl.Config.with_columns_kwargs = True
+
+    dx = df.with_columns(
+        d=pl.col("a") * pl.col("b"),
+        e=~pl.col("c"),
+        f=srs_unnamed,
+    )
+    assert_frame_equal(dx, expected)
+
+    # mixed
+    dx = df.with_columns(
+        [(pl.col("a") * pl.col("b")).alias("d")],
+        e=~pl.col("c"),
+        f=srs_unnamed,
+    )
+    assert_frame_equal(dx, expected)
+
+    # at least one of exprs/**named_exprs required
+    with pytest.raises(ValueError):
+        _ = df.with_columns()
