@@ -1,34 +1,35 @@
+from __future__ import annotations
+
 from datetime import date, datetime, time
 
 import pandas as pd
-from test_series import verify_series_and_expr_api
 
 import polars as pl
-from polars import testing
+from polars.testing import assert_series_equal, verify_series_and_expr_api
 
 
 def test_list_arr_get() -> None:
     a = pl.Series("a", [[1, 2, 3], [4, 5], [6, 7, 8, 9]])
     out = a.arr.get(0)
     expected = pl.Series("a", [1, 4, 6])
-    testing.assert_series_equal(out, expected)
+    assert_series_equal(out, expected)
     out = a.arr.first()
-    testing.assert_series_equal(out, expected)
+    assert_series_equal(out, expected)
     out = pl.select(pl.lit(a).arr.first()).to_series()
-    testing.assert_series_equal(out, expected)
+    assert_series_equal(out, expected)
 
     out = a.arr.get(-1)
     expected = pl.Series("a", [3, 5, 9])
-    testing.assert_series_equal(out, expected)
+    assert_series_equal(out, expected)
     out = a.arr.last()
-    testing.assert_series_equal(out, expected)
+    assert_series_equal(out, expected)
     out = pl.select(pl.lit(a).arr.last()).to_series()
-    testing.assert_series_equal(out, expected)
+    assert_series_equal(out, expected)
 
     a = pl.Series("a", [[1, 2, 3], [4, 5], [6, 7, 8, 9]])
     out = a.arr.get(-3)
     expected = pl.Series("a", [1, None, 7])
-    testing.assert_series_equal(out, expected)
+    assert_series_equal(out, expected)
 
     assert pl.DataFrame(
         {"a": [[1], [2], [3], [4, 5, 6], [7, 8, 9], [None, 11]]}
@@ -49,10 +50,10 @@ def test_contains() -> None:
     a = pl.Series("a", [[1, 2, 3], [2, 5], [6, 7, 8, 9]])
     out = a.arr.contains(2)
     expected = pl.Series("a", [True, True, False])
-    testing.assert_series_equal(out, expected)
+    assert_series_equal(out, expected)
 
     out = pl.select(pl.lit(a).arr.contains(2)).to_series()
-    testing.assert_series_equal(out, expected)
+    assert_series_equal(out, expected)
 
 
 def test_dtype() -> None:
@@ -169,7 +170,7 @@ def test_list_append() -> None:
     out = df.select([pl.col("a").arr.concat([1, 4])])
     assert out["a"][0].to_list() == [1, 2, 1, 4]
 
-    out_s = df["a"].arr.concat(([4, 1]))
+    out_s = df["a"].arr.concat([4, 1])
     assert out_s[0].to_list() == [1, 2, 4, 1]
 
 
